/**
 * \file src/gui/xic_view/xic_box/xicbox.cpp
 * \date 12/1/2018
 * \author Olivier Langella
 * \brief XIC box widget
 */
/*******************************************************************************
* Copyright (c) 2018 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "xicbox.h"
#include "ui_xic_box.h"
#include <QMessageBox>
#include <pappsomspp/pappsoexception.h>
#include "../xicworkerthread.h"
#include "../../project_view/projectwindow.h"

XicBox::XicBox(XicWindow *parent):
    QWidget(parent),
    ui(new Ui::XicBox)
{
    _p_xic_window = parent;
    ui->setupUi(this);

    XicWorkerThread * p_worker = new XicWorkerThread();
    p_worker->moveToThread(&_xic_thread);
    _xic_thread.start();

    //ui->xic_list_widget->setLayout(new QVBoxLayout(ui->xic_list_widget));

#if QT_VERSION >= 0x050000
    // Qt5 code
    connect(this, &XicBox::loadXic, p_worker, &XicWorkerThread::doXicLoad);
    connect(p_worker, &XicWorkerThread::xicLoaded,this, &XicBox::setXic);
    connect(p_worker, &XicWorkerThread::operationFailed, this, &XicBox::error);


    connect(this, &XicBox::computeIsotopeMassList, p_worker, &XicWorkerThread::doComputeIsotopeMassList);
    connect(p_worker, &XicWorkerThread::isotopeMassListComputed,this, &XicBox::setIsotopeMassList);

#else
// Qt4 code
#endif


}

XicBox::~XicBox()
{
    qDebug() << "XicBox::~XicBox";
    _xic_thread.quit();
    _xic_thread.wait();
}

void XicBox::remove() {
    _p_xic_window->removeXicBox(this);
}

void XicBox::error(QString error_message) {
    QMessageBox::warning(this,
                         tr("Error trying to save spectrum to SVG file :"), error_message);

}
void XicBox::setPeptideEvidence(const PeptideEvidence * p_peptide_evidence) {

    _p_peptide_evidence = p_peptide_evidence;
    ui->peptide_label->setText(_p_peptide_evidence->getPeptideXtpSp().get()->toString());
    ui->msrun_label->setText(_p_peptide_evidence->getMsRunP()->getFilename());
    ui->charge_label->setText(QString("%1").arg(_p_peptide_evidence->getCharge()));
    ui->mz_label->setText(QString::number(_p_peptide_evidence->getTheoreticalMz(), 'f', 4));

    //get same xic peptide evidence (msrun, peptide, charge)
    //p_projet
    _peptide_evidence_list.clear();
    _p_xic_window->getProjectWindow()->getProjectP()->getSameXicPeptideEvidenceList(p_peptide_evidence, _peptide_evidence_list);

    emit computeIsotopeMassList(_p_peptide_evidence->getPeptideXtpSp(), _p_peptide_evidence->getCharge(), _p_xic_window->getXicExtractPrecision(), 0.8);

}


void XicBox::setXic(std::vector< pappso::XicSp> xic_sp_list) {
    qDebug() << "XicBox::setXic begin";

    //pappso::XicWidget * xic_widget = new pappso::XicWidget(this);
    //ui->xic_list_widget->layout()->addWidget(xic_widget);
    for (int i=0; i < xic_sp_list.size(); i++) {
        qDebug() << "XicBox::setXic xic " << xic_sp_list[i].get();
        if (xic_sp_list[i].get() == nullptr) {
            throw new pappso::PappsoException("Error in XicBox::setXic:\n xic_sp_list[i].get() == nullptr");
        }
        qDebug() << "XicBox::setXic xic_sp_list[i].size() " << xic_sp_list[i].get()->size();
        ui->xic_widget->addXicSp(xic_sp_list[i]);
        QString isotope_name = QString("+%1").arg(_isotope_mass_list[i].get()->getIsotopeNumber());
        if (_isotope_mass_list[i].get()->getIsotopeRank() > 1) {
            isotope_name = QString("+%1 [%2]").arg(_isotope_mass_list[i].get()->getIsotopeNumber()).arg(_isotope_mass_list[i].get()->getIsotopeRank());
        }
        ui->xic_widget->setName(xic_sp_list[i].get(), isotope_name);

        if (_isotope_mass_list[i].get()->getIsotopeNumber() == 0) {
            for (const PeptideEvidence * peptide_evidence:_peptide_evidence_list) {
                qDebug() << "XicBox::setXic peptide_evidence " << peptide_evidence->getRetentionTime();
                ui->xic_widget->addMsMsEvent(xic_sp_list[i].get(), peptide_evidence->getRetentionTime());
            }
        }
    }
    qDebug() << "XicBox::setXic rescale" ;
    ui->xic_widget->rescale();
    qDebug() << "XicBox::setXic plot" ;
    ui->xic_widget->plot();

    /*
    if (_isotope_mass_list.size() > _xic_widget_list.size()) {
        emit loadXic(_p_peptide_evidence->getMsRunP(), _isotope_mass_list[_xic_widget_list.size()].get()->getMz(), _p_xic_window->getXicExtractPrecision(), XicExtractMethod::max);
    }
    */

    qDebug() << "XicBox::setXic end";

}


void XicBox::setIsotopeMassList(std::vector<pappso::PeptideNaturalIsotopeAverageSp> isotope_mass_list) {
    _isotope_mass_list = isotope_mass_list;

    std::vector<pappso::mz> mass_list;
    for (pappso::PeptideNaturalIsotopeAverageSp & natural_isotope_average: isotope_mass_list) {
        mass_list.push_back(natural_isotope_average.get()->getMz());
    }
    emit loadXic(_p_peptide_evidence->getMsRunP(), mass_list, _p_xic_window->getXicExtractPrecision(), XicExtractMethod::max);


}
